# coding:utf-8
import json
import threading

from volcengine.ApiInfo import ApiInfo
from volcengine.Credentials import Credentials
from volcengine.ServiceInfo import ServiceInfo
from volcengine.base.Service import Service
from volcengine.const.Const import REGION_CN_NORTH1

LIVE_SERVICE_VERSION = "2020-08-01"
service_info_map = {
    REGION_CN_NORTH1: ServiceInfo("live.volcengineapi.com", {'Accept': 'application/json', },
                                  Credentials('', '', "live", REGION_CN_NORTH1), 5, 5, "https"),
}

api_info = {
    "ListCommonTransPresetDetail": ApiInfo("POST", "/",
                                           {"Action": "ListCommonTransPresetDetail", "Version": LIVE_SERVICE_VERSION},
                                           {}, {}),
    "UpdateCallback": ApiInfo("POST", "/", {"Action": "UpdateCallback", "Version": LIVE_SERVICE_VERSION}, {}, {}),
    "DescribeCallback": ApiInfo("POST", "/", {"Action": "DescribeCallback", "Version": LIVE_SERVICE_VERSION}, {},
                                {}),
    "DeleteCallback": ApiInfo("POST", "/", {"Action": "DeleteCallback", "Version": LIVE_SERVICE_VERSION},
                              {}, {}),

    "CreateDomain": ApiInfo("POST", "/", {"Action": "CreateDomain", "Version": LIVE_SERVICE_VERSION}, {}, {}),

    "DeleteDomain": ApiInfo("POST", "/", {"Action": "DeleteDomain", "Version": LIVE_SERVICE_VERSION},
                            {},
                            {}),
    "ListDomainDetail": ApiInfo("POST", "/",
                                {"Action": "ListDomainDetail", "Version": LIVE_SERVICE_VERSION},
                                {}, {}),
    "DescribeDomain": ApiInfo("POST", "/",
                              {"Action": "DescribeDomain", "Version": LIVE_SERVICE_VERSION},
                              {}, {}),
    "EnableDomain": ApiInfo("POST", "/",
                            {"Action": "EnableDomain", "Version": LIVE_SERVICE_VERSION},
                            {}, {}),
    "DisableDomain": ApiInfo("POST", "/",
                             {"Action": "DisableDomain", "Version": LIVE_SERVICE_VERSION},
                             {}, {}),
    "ManagerPullPushDomainBind": ApiInfo("POST", "/",
                                         {"Action": "ManagerPullPushDomainBind", "Version": LIVE_SERVICE_VERSION},
                                         {},
                                         {}),
    "UpdateAuthKey": ApiInfo("POST", "/", {"Action": "UpdateAuthKey", "Version": LIVE_SERVICE_VERSION}, {}, {}),
    "DescribeAuth": ApiInfo("POST", "/", {"Action": "DescribeAuth", "Version": LIVE_SERVICE_VERSION}, {}, {}),
    "ForbidStream": ApiInfo("POST", "/", {"Action": "ForbidStream", "Version": LIVE_SERVICE_VERSION}, {},
                            {}),
    "ResumeStream": ApiInfo("POST", "/", {"Action": "ResumeStream", "Version": LIVE_SERVICE_VERSION}, {},
                            {}),
    "ListCert": ApiInfo("POST", "/", {"Action": "ListCert", "Version": LIVE_SERVICE_VERSION}, {},
                        {}),
    "CreateCert": ApiInfo("POST", "/", {"Action": "CreateCert", "Version": LIVE_SERVICE_VERSION}, {},
                          {}),
    "UpdateCert": ApiInfo("POST", "/", {"Action": "UpdateCert", "Version": LIVE_SERVICE_VERSION}, {},
                          {}),
    "BindCert": ApiInfo("POST", "/", {"Action": "BindCert", "Version": LIVE_SERVICE_VERSION}, {},
                        {}),
    "UnbindCert": ApiInfo("POST", "/", {"Action": "UnbindCert", "Version": LIVE_SERVICE_VERSION}, {},
                          {}),
    "DeleteCert": ApiInfo("POST", "/", {"Action": "DeleteCert", "Version": LIVE_SERVICE_VERSION}, {},
                          {}),
    "UpdateReferer": ApiInfo("POST", "/", {"Action": "UpdateReferer", "Version": LIVE_SERVICE_VERSION}, {},
                             {}),
    "DeleteReferer": ApiInfo("POST", "/", {"Action": "DeleteReferer", "Version": LIVE_SERVICE_VERSION}, {},
                             {}),
    "DescribeReferer": ApiInfo("POST", "/", {"Action": "DescribeReferer", "Version": LIVE_SERVICE_VERSION}, {},
                               {}),
    "CreateRecordPreset": ApiInfo("POST", "/", {"Action": "CreateRecordPreset", "Version": LIVE_SERVICE_VERSION}, {},
                                  {}),
    "UpdateRecordPreset": ApiInfo("POST", "/", {"Action": "UpdateRecordPreset", "Version": LIVE_SERVICE_VERSION}, {},
                                  {}),
    "DeleteRecordPreset": ApiInfo("POST", "/", {"Action": "DeleteRecordPreset", "Version": LIVE_SERVICE_VERSION}, {},
                                  {}),
    "ListVhostRecordPreset": ApiInfo("POST", "/", {"Action": "ListVhostRecordPreset", "Version": LIVE_SERVICE_VERSION},
                                     {},
                                     {}),
    "CreateTranscodePreset": ApiInfo("POST", "/", {"Action": "CreateTranscodePreset", "Version": LIVE_SERVICE_VERSION},
                                     {},
                                     {}),
    "UpdateTranscodePreset": ApiInfo("POST", "/", {"Action": "UpdateTranscodePreset", "Version": LIVE_SERVICE_VERSION},
                                     {},
                                     {}),
    "DeleteTranscodePreset": ApiInfo("POST", "/", {"Action": "DeleteTranscodePreset", "Version": LIVE_SERVICE_VERSION},
                                     {},
                                     {}),
    "ListVhostTransCodePreset": ApiInfo("POST", "/",
                                        {"Action": "ListVhostTransCodePreset", "Version": LIVE_SERVICE_VERSION}, {},
                                        {}),
    "CreateSnapshotPreset": ApiInfo("POST", "/", {"Action": "CreateSnapshotPreset", "Version": LIVE_SERVICE_VERSION},
                                    {},
                                    {}),
    "UpdateSnapshotPreset": ApiInfo("POST", "/", {"Action": "UpdateSnapshotPreset", "Version": LIVE_SERVICE_VERSION},
                                    {},
                                    {}),
    "DeleteSnapshotPreset": ApiInfo("POST", "/", {"Action": "DeleteSnapshotPreset", "Version": LIVE_SERVICE_VERSION},
                                    {},
                                    {}),
    "ListVhostSnapshotPreset": ApiInfo("POST", "/",
                                       {"Action": "ListVhostSnapshotPreset", "Version": LIVE_SERVICE_VERSION}, {},
                                       {}),
    "DescribeLiveBandwidthData": ApiInfo("POST", "/",
                                         {"Action": "DescribeLiveBandwidthData", "Version": LIVE_SERVICE_VERSION}, {},
                                         {}),
    "DescribeLiveTrafficData": ApiInfo("POST", "/",
                                       {"Action": "DescribeLiveTrafficData", "Version": LIVE_SERVICE_VERSION}, {},
                                       {}),
    "DescribeLiveP95PeakBandwidthData": ApiInfo("POST", "/",
                                                {"Action": "DescribeLiveP95PeakBandwidthData",
                                                 "Version": LIVE_SERVICE_VERSION}, {},
                                                {}),
    "DescribeRecordData": ApiInfo("POST", "/",
                                  {"Action": "DescribeRecordData", "Version": LIVE_SERVICE_VERSION}, {},
                                  {}),
    "DescribeTranscodeData": ApiInfo("POST", "/",
                                     {"Action": "DescribeTranscodeData", "Version": LIVE_SERVICE_VERSION}, {},
                                     {}),
    "DescribeSnapshotData": ApiInfo("POST", "/",
                                    {"Action": "DescribeSnapshotData", "Version": LIVE_SERVICE_VERSION}, {},
                                    {}),

    "DescribeLiveDomainLog": ApiInfo("GET", "/",
                                     {"Action": "DescribeLiveDomainLog", "Version": LIVE_SERVICE_VERSION}, {},
                                     {}),
    "DescribePushStreamMetrics": ApiInfo("POST", "/",
                                         {"Action": "DescribePushStreamMetrics", "Version": LIVE_SERVICE_VERSION}, {},
                                         {}),
    "DescribeLiveStreamSessions": ApiInfo("POST", "/",
                                          {"Action": "DescribeLiveStreamSessions", "Version": LIVE_SERVICE_VERSION}, {},
                                          {}),
    "DescribePlayResponseStatusStat": ApiInfo("POST", "/",
                                              {"Action": "DescribePlayResponseStatusStat",
                                               "Version": LIVE_SERVICE_VERSION}, {},
                                              {}),
    "DescribeLiveMetricTrafficData": ApiInfo("POST", "/",
                                             {"Action": "DescribeLiveMetricTrafficData",
                                              "Version": LIVE_SERVICE_VERSION}, {},
                                             {}),
    "DescribeLiveMetricBandwidthData": ApiInfo("POST", "/",
                                               {"Action": "DescribeLiveMetricBandwidthData",
                                                "Version": LIVE_SERVICE_VERSION}, {},
                                               {}),
    "DescribePlayStreamList": ApiInfo("GET", "/",
                                      {"Action": "DescribePlayStreamList",
                                       "Version": LIVE_SERVICE_VERSION}, {},
                                      {}),
    "DescribePullToPushBandwidthData":ApiInfo("POST", "/",
                                               {"Action": "DescribePullToPushBandwidthData",
                                                "Version": LIVE_SERVICE_VERSION}, {},
                                               {}),
}


class LiveService(Service):
    _instance_lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(LiveService, "_instance"):
            with LiveService._instance_lock:
                if not hasattr(LiveService, "_instance"):
                    LiveService._instance = object.__new__(cls)
        return LiveService._instance

    def __init__(self, region=REGION_CN_NORTH1):
        self.service_info = LiveService.get_service_info(region)
        self.api_info = LiveService.get_api_info()
        super(LiveService, self).__init__(self.service_info, self.api_info)

    @staticmethod
    def get_service_info(region_name):
        service_info = service_info_map.get(region_name, None)
        if not service_info:
            raise Exception('do not support region %s' % region_name)
        return service_info

    @staticmethod
    def get_api_info():
        return api_info

    def list_common_trans_preset_detail(self, params):
        action = "ListCommonTransPresetDetail"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def update_callback(self, params):
        action = "UpdateCallback"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_callback(self, params):
        action = "DescribeCallback"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def delete_callback(self, params):
        action = "DeleteCallback"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def create_domain(self, params):
        action = "CreateDomain"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def delete_domain(self, params):
        action = "DeleteDomain"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def list_domain_detail(self, params):
        action = "ListDomainDetail"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_domain(self, params):
        action = "DescribeDomain"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def enable_domain(self, params):
        action = "EnableDomain"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def disable_domain(self, params):
        action = "DisableDomain"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def manager_pull_push_domain_bind(self, params):
        action = "ManagerPullPushDomainBind"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def update_auth_key(self, params):
        action = "UpdateAuthKey"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_auth(self, params):
        action = "DescribeAuth"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def forbid_stream(self, params):
        action = "ForbidStream"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def resume_stream(self, params):
        action = "ResumeStream"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def list_cert(self, params):
        action = "ListCert"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def create_cert(self, params):
        action = "CreateCert"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def update_cert(self, params):
        action = "UpdateCert"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def bind_cert(self, params):
        action = "BindCert"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def un_bind_cert(self, params):
        action = "UnbindCert"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def delete_cert(self, params):
        action = "DeleteCert"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def update_referer(self, params):
        action = "UpdateReferer"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def delete_referer(self, params):
        action = "DeleteReferer"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_referer(self, params):
        action = "DescribeReferer"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def create_record_preset(self, params):
        action = "CreateRecordPreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def update_record_preset(self, params):
        action = "UpdateRecordPreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def delete_record_preset(self, params):
        action = "DeleteRecordPreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def list_vhost_record_preset(self, params):
        action = "ListVhostRecordPreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def create_transcode_preset(self, params):
        action = "CreateTranscodePreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def update_transcode_preset(self, params):
        action = "UpdateTranscodePreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def delete_transcode_preset(self, params):
        action = "DeleteTranscodePreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def list_vhost_transcode_preset(self, params):
        action = "ListVhostTransCodePreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def create_snapshot_preset(self, params):
        action = "CreateSnapshotPreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def update_snapshot_preset(self, params):
        action = "UpdateSnapshotPreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def delete_snapshot_preset(self, params):
        action = "DeleteSnapshotPreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def list_vhost_snapshot_preset(self, params):
        action = "ListVhostSnapshotPreset"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_live_bandwidth_data(self, params):
        action = "DescribeLiveBandwidthData"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_live_traffic_data(self, params):
        action = "DescribeLiveTrafficData"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_live_P95Peak_bandwidth_data(self, params):
        action = "DescribeLiveP95PeakBandwidthData"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_record_data(self, params):
        action = "DescribeRecordData"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_transcode_data(self, params):
        action = "DescribeTranscodeData"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_snapshot_data(self, params):
        action = "DescribeSnapshotData"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_live_domain_log(self, params):
        action = "DescribeLiveDomainLog"
        res = self.get(action, params)
        # res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_push_stream_metrics(self, params):
        action = "DescribePushStreamMetrics"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_live_stream_sessions(self, params):
        action = "DescribeLiveStreamSessions"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_play_response_status_stat(self, params):
        action = "DescribePlayResponseStatusStat"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_live_metric_traffic_data(self, params):
        action = "DescribeLiveMetricTrafficData"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_live_metric_bandwidth_data(self, params):
        action = "DescribeLiveMetricBandwidthData"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_play_stream_list(self, params):
        action = "DescribePlayStreamList"
        res = self.get(action, params)
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json

    def describe_pull_to_push_bandwidth_data(self, params):
        action = "DescribePullToPushBandwidthData"
        res = self.json(action, dict(), json.dumps(params))
        if res == '':
            raise Exception("%s: empty response" % action)
        res_json = json.loads(res)
        return res_json



# coding:utf-8
from __future__ import print_function

from volcengine.vod.VodService import VodService
from volcengine.vod.models.request.request_vod_pb2 import VodGetSubtitleInfoListRequest

if __name__ == '__main__':
    vod_service = VodService()
    # call below method if you dont set ak and sk in $HOME/.vcloud/config
    # vod_service.set_ak('your ak')
    # vod_service.set_sk('your sk')

    try:
        req11 = VodGetSubtitleInfoListRequest()
        req11.Vid = 'vid'

        token = vod_service.get_subtitle_auth_token(req11, 60)
    except Exception:
        raise
    else:
        print(token)

    print('*' * 100)


# coding:utf-8
import json
import threading

from volcengine.ApiInfo import ApiInfo
from volcengine.Credentials import Credentials
from volcengine.base.Service import Service
from volcengine.ServiceInfo import ServiceInfo


class RtcService(Service):
    _instance_lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(RtcService, "_instance"):
            with RtcService._instance_lock:
                if not hasattr(RtcService, "_instance"):
                    RtcService._instance = object.__new__(cls)
        return RtcService._instance

    def __init__(self):
        self.service_info = RtcService.get_service_info()
        self.api_info = RtcService.get_api_info()
        super(RtcService, self).__init__(self.service_info, self.api_info)

    @staticmethod
    def get_service_info():
        service_info = ServiceInfo("rtc.volcengineapi.com", {'Accept': 'application/json'},
                                   Credentials('', '', 'rtc', 'cn-north-1'), 5, 5)
        return service_info

    @staticmethod
    def get_api_info():
        api_info = {
            "ListRoomInformation": ApiInfo(
                "GET", "/", {"Action": "ListRoomInformation", "Version": "2020-12-01"}, {}, {}
            ),
            "ListIndicators": ApiInfo("POST", "/", {"Action": "ListIndicators", "Version": "2020-12-01"}, {}, {}),
        }
        return api_info

    def list_room_information(self, params):
        res = self.get("ListRoomInformation", params)
        if res == '':
            raise Exception("ListRoomInformation: empty response")
        res_json = json.loads(res)
        return res_json

    def list_indicators(self, body):
        res = self.json("ListIndicators", {}, body)
        if res == '':
            raise Exception("ListIndicators: empty response")
        res_json = json.loads(res)
        return res_json



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: vod/business/vod_cdn.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from volcengine.vod.models.business import vod_common_pb2 as vod_dot_business_dot_vod__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1avod/business/vod_cdn.proto\x12\x1eVolcengine.Vod.Models.Business\x1a\x1dvod/business/vod_common.proto\"\xcd\x01\n\x13VodDomainConfigInfo\x12\x11\n\tSpaceName\x18\x01 \x01(\t\x12P\n\x10PlayInstanceInfo\x18\x02 \x01(\x0b\x32\x36.Volcengine.Vod.Models.Business.VodDomainInstanceInfos\x12Q\n\x11ImageInstanceInfo\x18\x03 \x01(\x0b\x32\x36.Volcengine.Vod.Models.Business.VodDomainInstanceInfos\"\xb5\x01\n\x16VodDomainInstanceInfos\x12L\n\rByteInstances\x18\x01 \x03(\x0b\x32\x35.Volcengine.Vod.Models.Business.VodDomainInstanceInfo\x12M\n\x0eOtherInstances\x18\x02 \x03(\x0b\x32\x35.Volcengine.Vod.Models.Business.VodDomainInstanceInfo\"\x9a\x01\n\x15VodDomainInstanceInfo\x12\x12\n\nInstanceId\x18\x01 \x01(\t\x12?\n\x07\x44omains\x18\x02 \x03(\x0b\x32..Volcengine.Vod.Models.Business.VodDomainoInfo\x12\x16\n\x0e\x43\x61nSelfEditing\x18\x03 \x01(\x08\x12\x14\n\x0c\x43onfigStatus\x18\x04 \x01(\t\"\xe1\x01\n\x0eVodDomainoInfo\x12\x0e\n\x06\x44omain\x18\x01 \x01(\t\x12\r\n\x05\x43name\x18\x02 \x01(\t\x12\x14\n\x0c\x43onfigStatus\x18\x03 \x01(\t\x12\x13\n\x0b\x43nameStatus\x18\x04 \x01(\t\x12\x0e\n\x06Status\x18\x05 \x01(\t\x12M\n\x0b\x43\x65rtificate\x18\x06 \x01(\x0b\x32\x38.Volcengine.Vod.Models.Business.VodDomainCertificateInfo\x12\x12\n\nCreateTime\x18\x07 \x01(\t\x12\x12\n\nUpdateTime\x18\x08 \x01(\t\"\x8f\x01\n\x18VodDomainCertificateInfo\x12\x15\n\rCertificateId\x18\x01 \x01(\t\x12\x17\n\x0f\x43\x65rtificateName\x18\x02 \x01(\t\x12\x16\n\x0e\x43\x65rtificatePub\x18\x03 \x01(\t\x12\x16\n\x0e\x43\x65rtificatePri\x18\x04 \x01(\t\x12\x13\n\x0bHttpsStatus\x18\x05 \x01(\t\"(\n\x16VodCreateCdnTaskResult\x12\x0e\n\x06TaskId\x18\x01 \x01(\t\"x\n\x0eVodContentInfo\x12\x0e\n\x06ItemId\x18\x01 \x01(\t\x12\x0b\n\x03Url\x18\x02 \x01(\t\x12\x0e\n\x06Status\x18\x03 \x01(\t\x12\x10\n\x08TaskType\x18\x04 \x01(\t\x12\x17\n\x0f\x43reateTimestamp\x18\x05 \x01(\x05\x12\x0e\n\x06TaskId\x18\x06 \x01(\t\"\x8f\x01\n\x10VodCdnTaskResult\x12\x12\n\nTotalCount\x18\x01 \x01(\x05\x12\x0f\n\x07PageNum\x18\x02 \x01(\x05\x12\x10\n\x08PageSize\x18\x03 \x01(\x05\x12\x44\n\x0c\x43ontentInfos\x18\x04 \x03(\x0b\x32..Volcengine.Vod.Models.Business.VodContentInfo\"\x7f\n\x16VodCdnAccessLogElement\x12\x13\n\x0b\x44ownloadUrl\x18\x01 \x01(\t\x12\x10\n\x08\x46ileSize\x18\x02 \x01(\x03\x12\x10\n\x08\x46ileName\x18\x03 \x01(\t\x12\x16\n\x0eStartTimestamp\x18\x04 \x01(\x05\x12\x14\n\x0c\x45ndTimestamp\x18\x05 \x01(\x05\"n\n\x13VodCdnAccessLogInfo\x12\x0e\n\x06\x44omain\x18\x01 \x01(\t\x12G\n\x07LogList\x18\x02 \x03(\x0b\x32\x36.Volcengine.Vod.Models.Business.VodCdnAccessLogElement\"^\n\x19VodListCdnAccessLogResult\x12\x41\n\x04Logs\x18\x01 \x03(\x0b\x32\x33.Volcengine.Vod.Models.Business.VodCdnAccessLogInfo\"B\n\x19VodCdnTopAccessUrlElement\x12\x0b\n\x03Url\x18\x01 \x01(\t\x12\n\n\x02Pv\x18\x02 \x01(\x03\x12\x0c\n\x04\x46lux\x18\x03 \x01(\x03\"k\n\x1cVodListCdnTopAccessUrlResult\x12K\n\x08UrlInfos\x18\x01 \x03(\x0b\x32\x39.Volcengine.Vod.Models.Business.VodCdnTopAccessUrlElement\"3\n\x10VodBandwidthData\x12\x0c\n\x04Time\x18\x01 \x01(\t\x12\x11\n\tBandwidth\x18\x02 \x01(\x01\"\x8c\x02\n\'VodDescribeVodDomainBandwidthDataResult\x12\x12\n\nDomainList\x18\x01 \x03(\t\x12\x11\n\tStartTime\x18\x02 \x01(\t\x12\x0f\n\x07\x45ndTime\x18\x03 \x01(\t\x12\x13\n\x0b\x41ggregation\x18\x04 \x01(\x05\x12\x15\n\rBandwidthType\x18\x05 \x01(\t\x12\x15\n\rPeakBandwidth\x18\x06 \x01(\x01\x12\x19\n\x11PeakBandwidthTime\x18\x07 \x01(\t\x12K\n\x11\x42\x61ndwidthDataList\x18\x08 \x03(\x0b\x32\x30.Volcengine.Vod.Models.Business.VodBandwidthData\"\x80\x01\n\x14VodCdnStatisticsData\x12\x0c\n\x04Name\x18\x01 \x01(\t\x12\x0e\n\x06Metric\x18\x02 \x01(\t\x12\x10\n\x08\x44\x61taType\x18\x03 \x01(\t\x12\x38\n\x06Points\x18\x04 \x03(\x0b\x32(.Volcengine.Vod.Models.Business.VodPoint\"\x80\x01\n\x1cVodCdnStatisticsCommonResult\x12\x43\n\x05\x44\x61tas\x18\x01 \x03(\x0b\x32\x34.Volcengine.Vod.Models.Business.VodCdnStatisticsData\x12\x1b\n\x13NoPermissionDomains\x18\x02 \x03(\t\"H\n\x0cVodCdnIpInfo\x12\n\n\x02Ip\x18\x01 \x01(\t\x12\r\n\x05\x43\x64nIp\x18\x02 \x01(\x08\x12\x10\n\x08Location\x18\x03 \x01(\t\x12\x0b\n\x03Isp\x18\x04 \x01(\t\"V\n\x17VodDescribeIpInfoResult\x12;\n\x05Infos\x18\x01 \x03(\x0b\x32,.Volcengine.Vod.Models.Business.VodCdnIpInfoB\xca\x01\n)com.volcengine.service.vod.model.businessB\x06VodCdnP\x01ZAgithub.com/volcengine/volc-sdk-golang/service/vod/models/business\xa0\x01\x01\xd8\x01\x01\xc2\x02\x00\xca\x02 Volc\\Service\\Vod\\Models\\Business\xe2\x02#Volc\\Service\\Vod\\Models\\GPBMetadatab\x06proto3')



_VODDOMAINCONFIGINFO = DESCRIPTOR.message_types_by_name['VodDomainConfigInfo']
_VODDOMAININSTANCEINFOS = DESCRIPTOR.message_types_by_name['VodDomainInstanceInfos']
_VODDOMAININSTANCEINFO = DESCRIPTOR.message_types_by_name['VodDomainInstanceInfo']
_VODDOMAINOINFO = DESCRIPTOR.message_types_by_name['VodDomainoInfo']
_VODDOMAINCERTIFICATEINFO = DESCRIPTOR.message_types_by_name['VodDomainCertificateInfo']
_VODCREATECDNTASKRESULT = DESCRIPTOR.message_types_by_name['VodCreateCdnTaskResult']
_VODCONTENTINFO = DESCRIPTOR.message_types_by_name['VodContentInfo']
_VODCDNTASKRESULT = DESCRIPTOR.message_types_by_name['VodCdnTaskResult']
_VODCDNACCESSLOGELEMENT = DESCRIPTOR.message_types_by_name['VodCdnAccessLogElement']
_VODCDNACCESSLOGINFO = DESCRIPTOR.message_types_by_name['VodCdnAccessLogInfo']
_VODLISTCDNACCESSLOGRESULT = DESCRIPTOR.message_types_by_name['VodListCdnAccessLogResult']
_VODCDNTOPACCESSURLELEMENT = DESCRIPTOR.message_types_by_name['VodCdnTopAccessUrlElement']
_VODLISTCDNTOPACCESSURLRESULT = DESCRIPTOR.message_types_by_name['VodListCdnTopAccessUrlResult']
_VODBANDWIDTHDATA = DESCRIPTOR.message_types_by_name['VodBandwidthData']
_VODDESCRIBEVODDOMAINBANDWIDTHDATARESULT = DESCRIPTOR.message_types_by_name['VodDescribeVodDomainBandwidthDataResult']
_VODCDNSTATISTICSDATA = DESCRIPTOR.message_types_by_name['VodCdnStatisticsData']
_VODCDNSTATISTICSCOMMONRESULT = DESCRIPTOR.message_types_by_name['VodCdnStatisticsCommonResult']
_VODCDNIPINFO = DESCRIPTOR.message_types_by_name['VodCdnIpInfo']
_VODDESCRIBEIPINFORESULT = DESCRIPTOR.message_types_by_name['VodDescribeIpInfoResult']
VodDomainConfigInfo = _reflection.GeneratedProtocolMessageType('VodDomainConfigInfo', (_message.Message,), {
  'DESCRIPTOR' : _VODDOMAINCONFIGINFO,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodDomainConfigInfo)
  })
_sym_db.RegisterMessage(VodDomainConfigInfo)

VodDomainInstanceInfos = _reflection.GeneratedProtocolMessageType('VodDomainInstanceInfos', (_message.Message,), {
  'DESCRIPTOR' : _VODDOMAININSTANCEINFOS,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodDomainInstanceInfos)
  })
_sym_db.RegisterMessage(VodDomainInstanceInfos)

VodDomainInstanceInfo = _reflection.GeneratedProtocolMessageType('VodDomainInstanceInfo', (_message.Message,), {
  'DESCRIPTOR' : _VODDOMAININSTANCEINFO,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodDomainInstanceInfo)
  })
_sym_db.RegisterMessage(VodDomainInstanceInfo)

VodDomainoInfo = _reflection.GeneratedProtocolMessageType('VodDomainoInfo', (_message.Message,), {
  'DESCRIPTOR' : _VODDOMAINOINFO,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodDomainoInfo)
  })
_sym_db.RegisterMessage(VodDomainoInfo)

VodDomainCertificateInfo = _reflection.GeneratedProtocolMessageType('VodDomainCertificateInfo', (_message.Message,), {
  'DESCRIPTOR' : _VODDOMAINCERTIFICATEINFO,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodDomainCertificateInfo)
  })
_sym_db.RegisterMessage(VodDomainCertificateInfo)

VodCreateCdnTaskResult = _reflection.GeneratedProtocolMessageType('VodCreateCdnTaskResult', (_message.Message,), {
  'DESCRIPTOR' : _VODCREATECDNTASKRESULT,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodCreateCdnTaskResult)
  })
_sym_db.RegisterMessage(VodCreateCdnTaskResult)

VodContentInfo = _reflection.GeneratedProtocolMessageType('VodContentInfo', (_message.Message,), {
  'DESCRIPTOR' : _VODCONTENTINFO,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodContentInfo)
  })
_sym_db.RegisterMessage(VodContentInfo)

VodCdnTaskResult = _reflection.GeneratedProtocolMessageType('VodCdnTaskResult', (_message.Message,), {
  'DESCRIPTOR' : _VODCDNTASKRESULT,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodCdnTaskResult)
  })
_sym_db.RegisterMessage(VodCdnTaskResult)

VodCdnAccessLogElement = _reflection.GeneratedProtocolMessageType('VodCdnAccessLogElement', (_message.Message,), {
  'DESCRIPTOR' : _VODCDNACCESSLOGELEMENT,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodCdnAccessLogElement)
  })
_sym_db.RegisterMessage(VodCdnAccessLogElement)

VodCdnAccessLogInfo = _reflection.GeneratedProtocolMessageType('VodCdnAccessLogInfo', (_message.Message,), {
  'DESCRIPTOR' : _VODCDNACCESSLOGINFO,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodCdnAccessLogInfo)
  })
_sym_db.RegisterMessage(VodCdnAccessLogInfo)

VodListCdnAccessLogResult = _reflection.GeneratedProtocolMessageType('VodListCdnAccessLogResult', (_message.Message,), {
  'DESCRIPTOR' : _VODLISTCDNACCESSLOGRESULT,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodListCdnAccessLogResult)
  })
_sym_db.RegisterMessage(VodListCdnAccessLogResult)

VodCdnTopAccessUrlElement = _reflection.GeneratedProtocolMessageType('VodCdnTopAccessUrlElement', (_message.Message,), {
  'DESCRIPTOR' : _VODCDNTOPACCESSURLELEMENT,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodCdnTopAccessUrlElement)
  })
_sym_db.RegisterMessage(VodCdnTopAccessUrlElement)

VodListCdnTopAccessUrlResult = _reflection.GeneratedProtocolMessageType('VodListCdnTopAccessUrlResult', (_message.Message,), {
  'DESCRIPTOR' : _VODLISTCDNTOPACCESSURLRESULT,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodListCdnTopAccessUrlResult)
  })
_sym_db.RegisterMessage(VodListCdnTopAccessUrlResult)

VodBandwidthData = _reflection.GeneratedProtocolMessageType('VodBandwidthData', (_message.Message,), {
  'DESCRIPTOR' : _VODBANDWIDTHDATA,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodBandwidthData)
  })
_sym_db.RegisterMessage(VodBandwidthData)

VodDescribeVodDomainBandwidthDataResult = _reflection.GeneratedProtocolMessageType('VodDescribeVodDomainBandwidthDataResult', (_message.Message,), {
  'DESCRIPTOR' : _VODDESCRIBEVODDOMAINBANDWIDTHDATARESULT,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodDescribeVodDomainBandwidthDataResult)
  })
_sym_db.RegisterMessage(VodDescribeVodDomainBandwidthDataResult)

VodCdnStatisticsData = _reflection.GeneratedProtocolMessageType('VodCdnStatisticsData', (_message.Message,), {
  'DESCRIPTOR' : _VODCDNSTATISTICSDATA,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodCdnStatisticsData)
  })
_sym_db.RegisterMessage(VodCdnStatisticsData)

VodCdnStatisticsCommonResult = _reflection.GeneratedProtocolMessageType('VodCdnStatisticsCommonResult', (_message.Message,), {
  'DESCRIPTOR' : _VODCDNSTATISTICSCOMMONRESULT,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodCdnStatisticsCommonResult)
  })
_sym_db.RegisterMessage(VodCdnStatisticsCommonResult)

VodCdnIpInfo = _reflection.GeneratedProtocolMessageType('VodCdnIpInfo', (_message.Message,), {
  'DESCRIPTOR' : _VODCDNIPINFO,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodCdnIpInfo)
  })
_sym_db.RegisterMessage(VodCdnIpInfo)

VodDescribeIpInfoResult = _reflection.GeneratedProtocolMessageType('VodDescribeIpInfoResult', (_message.Message,), {
  'DESCRIPTOR' : _VODDESCRIBEIPINFORESULT,
  '__module__' : 'vod.business.vod_cdn_pb2'
  # @@protoc_insertion_point(class_scope:Volcengine.Vod.Models.Business.VodDescribeIpInfoResult)
  })
_sym_db.RegisterMessage(VodDescribeIpInfoResult)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n)com.volcengine.service.vod.model.businessB\006VodCdnP\001ZAgithub.com/volcengine/volc-sdk-golang/service/vod/models/business\240\001\001\330\001\001\302\002\000\312\002 Volc\\Service\\Vod\\Models\\Business\342\002#Volc\\Service\\Vod\\Models\\GPBMetadata'
  _VODDOMAINCONFIGINFO._serialized_start=94
  _VODDOMAINCONFIGINFO._serialized_end=299
  _VODDOMAININSTANCEINFOS._serialized_start=302
  _VODDOMAININSTANCEINFOS._serialized_end=483
  _VODDOMAININSTANCEINFO._serialized_start=486
  _VODDOMAININSTANCEINFO._serialized_end=640
  _VODDOMAINOINFO._serialized_start=643
  _VODDOMAINOINFO._serialized_end=868
  _VODDOMAINCERTIFICATEINFO._serialized_start=871
  _VODDOMAINCERTIFICATEINFO._serialized_end=1014
  _VODCREATECDNTASKRESULT._serialized_start=1016
  _VODCREATECDNTASKRESULT._serialized_end=1056
  _VODCONTENTINFO._serialized_start=1058
  _VODCONTENTINFO._serialized_end=1178
  _VODCDNTASKRESULT._serialized_start=1181
  _VODCDNTASKRESULT._serialized_end=1324
  _VODCDNACCESSLOGELEMENT._serialized_start=1326
  _VODCDNACCESSLOGELEMENT._serialized_end=1453
  _VODCDNACCESSLOGINFO._serialized_start=1455
  _VODCDNACCESSLOGINFO._serialized_end=1565
  _VODLISTCDNACCESSLOGRESULT._serialized_start=1567
  _VODLISTCDNACCESSLOGRESULT._serialized_end=1661
  _VODCDNTOPACCESSURLELEMENT._serialized_start=1663
  _VODCDNTOPACCESSURLELEMENT._serialized_end=1729
  _VODLISTCDNTOPACCESSURLRESULT._serialized_start=1731
  _VODLISTCDNTOPACCESSURLRESULT._serialized_end=1838
  _VODBANDWIDTHDATA._serialized_start=1840
  _VODBANDWIDTHDATA._serialized_end=1891
  _VODDESCRIBEVODDOMAINBANDWIDTHDATARESULT._serialized_start=1894
  _VODDESCRIBEVODDOMAINBANDWIDTHDATARESULT._serialized_end=2162
  _VODCDNSTATISTICSDATA._serialized_start=2165
  _VODCDNSTATISTICSDATA._serialized_end=2293
  _VODCDNSTATISTICSCOMMONRESULT._serialized_start=2296
  _VODCDNSTATISTICSCOMMONRESULT._serialized_end=2424
  _VODCDNIPINFO._serialized_start=2426
  _VODCDNIPINFO._serialized_end=2498
  _VODDESCRIBEIPINFORESULT._serialized_start=2500
  _VODDESCRIBEIPINFORESULT._serialized_end=2586
# @@protoc_insertion_point(module_scope)



# Code generated by protoc-gen-volcengine-sdk
# source: VodService
# DO NOT EDIT!
# coding:utf-8
from __future__ import print_function
from volcengine.Policy import *
from google.protobuf.json_format import *
from volcengine.vod.VodServiceConfig import VodServiceConfig
from retry import retry
from zlib import crc32
import os
import time
import datetime
from volcengine.util.Util import Util
from volcengine.vod.models.request.request_vod_pb2 import *
from volcengine.vod.models.response.response_vod_pb2 import *

MinChunkSize = 1024 * 1024 * 20
LargeFileSize = 1024 * 1024 * 1024


#
# Generated from protobuf service <code>VodService</code>
#
class VodService(VodServiceConfig):

    def get_private_drm_play_auth_token(self, request, expire):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            if expire > 0:
                params['X-Expires'] = str(expire)
            data = self.get_sign_url('GetPrivateDrmPlayAuth', params)
            return data
        except Exception as Argument:
            raise Argument

    def create_hls_drm_auth_token(self, auth_algorithm, expire_seconds):
        try:
            if expire_seconds == 0:
                raise Exception("invalid expire")
            deadline = int(time.mktime(datetime.datetime.now().timetuple())) + expire_seconds
            deadTime = datetime.datetime.utcfromtimestamp(deadline).strftime("%Y%m%dT%H%M%SZ")
            if sys.version_info[0] == 3:
                kDate = Util.hmac_sha256(bytes(self.service_info.credentials.sk, encoding='utf-8'), deadTime)
            else:
                kDate = Util.hmac_sha256(bytes(self.service_info.credentials.sk.encode('utf-8')), deadTime)
            kRegion = Util.hmac_sha256(kDate, self.service_info.credentials.region)
            kService = Util.hmac_sha256(kRegion, 'vod')
            kCredentials = Util.hmac_sha256(kService, 'request')
            key = Util.to_hex(kCredentials)
            signDataString = '&'.join([auth_algorithm, '2.0', str(deadline)])
            if auth_algorithm == 'HMAC-SHA1':
                if sys.version_info[0] == 3:
                    signBytes = Util.hmac_sha1(bytes(key, encoding='utf-8'), signDataString)
                else:
                    signBytes = Util.hmac_sha1(bytes(key.encode('utf-8')), signDataString)
            else:
                raise Exception('invalid authAlgorithm')
            sign = base64.b64encode(signBytes).decode('utf-8')
            token = ':'.join([auth_algorithm, '2.0', str(deadline), self.service_info.credentials.ak, sign])
            params = dict()
            params['DrmAuthToken'] = token
            params['X-Expires'] = str(expire_seconds)
            getAuth = self.get_sign_url("GetHlsDecryptionKey", params)
            return getAuth
        except Exception as e:
            raise e

    def get_sha1_hls_drm_auth_token(self, expire_seconds):
        try:
            return self.create_hls_drm_auth_token('HMAC-SHA1', expire_seconds)
        except Exception as Argument:
            raise Argument

    def get_play_auth_token(self, request, expire):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            if expire > 0:
                params['X-Expires'] = str(expire)
            token = self.get_sign_url('GetPlayInfo', params)
            ret = {'TokenVersion': 'V2', 'GetPlayInfoToken': token}
            data = json.dumps(ret)
        except Exception as Argument:
            raise Argument
        else:
            if sys.version_info[0] == 3:
                return base64.b64encode(data.encode('utf-8')).decode('utf-8')
            else:
                return base64.b64encode(data.decode('utf-8'))

    def get_subtitle_auth_token(self, request, expire):
        try:
            if request.Vid == "":
                raise Exception("Vid is None")
            params = {"Vid": request.Vid}
            params["Status"] = "Published"
            if expire > 0:
                params['X-Expires'] = str(expire)
            token = self.get_sign_url('GetSubtitleInfoList', params)
            ret = {'GetSubtitleAuthToken': token}
            data = json.dumps(ret)
        except Exception as Argument:
            raise Argument
        else:
            if sys.version_info[0] == 3:
                return base64.b64encode(data.encode('utf-8')).decode('utf-8')
            else:
                return base64.b64encode(data.decode('utf-8'))

    def upload_media(self, request):
        oid, session_key, avg_speed = self.upload_tob(request.SpaceName, request.FilePath, "", request.FileName)
        req = VodCommitUploadInfoRequest()
        req.SpaceName = request.SpaceName
        req.SessionKey = session_key
        req.Functions = request.Functions
        req.CallbackArgs = request.CallbackArgs
        resp = self.commit_upload_info(req)
        if resp.ResponseMetadata.Error.Code != '':
            print(resp.ResponseMetadata.RequestId)
            raise Exception(resp.ResponseMetadata.Error)
        return resp

    def upload_tob(self, space_name, file_path, file_type, file_name):
        if not os.path.isfile(file_path):
            raise Exception("no such file on file path")
        apply_req = VodApplyUploadInfoRequest()
        apply_req.SpaceName = space_name
        apply_req.FileType = file_type
        apply_req.FileName = file_name
        resp = self.apply_upload_info(apply_req)
        if resp.ResponseMetadata.Error.Code != '':
            print(resp.ResponseMetadata.RequestId)
            raise Exception(resp.ResponseMetadata.Error)
        upload_address = resp.Result.Data.UploadAddress
        oid = upload_address.StoreInfos[0].StoreUri
        session_key = upload_address.SessionKey
        auth = upload_address.StoreInfos[0].Auth
        host = upload_address.UploadHosts[0]
        start = time.time()
        file_size = os.path.getsize(file_path)
        if file_size < MinChunkSize:
            self.direct_upload(host, oid, auth, file_path)
        elif file_size > LargeFileSize:
            self.chunk_upload(file_path, host, oid, auth, file_size, True)
        else:
            self.chunk_upload(file_path, host, oid, auth, file_size, False)
        cost = (time.time() - start) * 1000
        file_size = os.path.getsize(file_path)
        avg_speed = float(file_size) / float(cost)
        return oid, session_key, avg_speed

    @retry(tries=3, delay=1, backoff=2)
    def direct_upload(self, host, oid, auth, file_path):
        with open(file_path, 'rb') as f:
            data = f.read()
            check_sum = crc32(data) & 0xFFFFFFFF
        check_sum = "%08x" % check_sum
        url = 'http://{}/{}'.format(host, oid)
        headers = {'Content-CRC32': check_sum, 'Authorization': auth}
        upload_status, resp = self.put(url, file_path, headers)
        if not upload_status:
            raise Exception("direct upload error")
        resp = json.loads(resp)
        if resp.get('success') is None or resp['success'] != 0:
            raise Exception("direct upload error")

    def chunk_upload(self, file_path, host, oid, auth, size, is_large_file):
        upload_id = self.init_upload_part(host, oid, auth, is_large_file)
        n = size // MinChunkSize
        last_num = n - 1
        parts = []
        with open(file_path, 'rb') as f:
            for i in range(0, last_num):
                data = f.read(MinChunkSize)
                part_number = i
                if is_large_file:
                    part_number = i + 1
                part = self.upload_part(host, oid, auth, upload_id, part_number, data, is_large_file)
                parts.append(part)
            data = f.read()
            if is_large_file:
                last_num = last_num + 1
            part = self.upload_part(host, oid, auth, upload_id, last_num, data, is_large_file)
            parts.append(part)
        return self.upload_merge_part(host, oid, auth, upload_id, parts, is_large_file)

    @retry(tries=3, delay=1, backoff=2)
    def init_upload_part(self, host, oid, auth, is_large_file):
        url = 'http://{}/{}?uploads'.format(host, oid)
        headers = {'Authorization': auth}
        if is_large_file:
            headers['X-Storage-Mode'] = 'gateway'
        upload_status, resp = self.put_data(url, None, headers)
        resp = json.loads(resp)
        if not upload_status:
            raise Exception("init upload error")
        if resp.get('success') is None or resp['success'] != 0:
            raise Exception("init upload error")
        return resp['payload']['uploadID']

    @retry(tries=3, delay=1, backoff=2)
    def upload_part(self, host, oid, auth, upload_id, part_number, data, is_large_file):
        url = 'http://{}/{}?partNumber={}&uploadID={}'.format(host, oid,
                                                              part_number, upload_id)
        check_sum = crc32(data) & 0xFFFFFFFF
        check_sum = "%08x" % check_sum
        headers = {'Content-CRC32': check_sum, 'Authorization': auth}
        if is_large_file:
            headers['X-Storage-Mode'] = 'gateway'
        upload_status, resp = self.put_data(url, data, headers)
        if not upload_status:
            raise Exception(url + json.dumps(resp))
        resp = json.loads(resp)
        if resp.get('success') is None or resp['success'] != 0:
            raise Exception("upload part error")
        return check_sum

    @staticmethod
    def generate_merge_body(check_sum_list):
        if len(check_sum_list) == 0:
            raise Exception('crc32 list empty')
        s = []
        for i in range(len(check_sum_list)):
            s.append('{}:{}'.format(i, check_sum_list[i]))
        comma = ','
        return comma.join(s)

    @retry(tries=3, delay=1, backoff=2)
    def upload_merge_part(self, host, oid, auth, upload_id, check_sum_list, is_large_file):
        url = 'http://{}/{}?uploadID={}'.format(host, oid, upload_id)
        data = self.generate_merge_body(check_sum_list)
        headers = {'Authorization': auth}
        if is_large_file:
            headers['X-Storage-Mode'] = 'gateway'
        upload_status, resp = self.put_data(url, data, headers)
        resp = json.loads(resp)
        if not upload_status:
            raise Exception("init upload error")
        if resp.get('success') is None or resp['success'] != 0:
            raise Exception("init upload error")

    def get_upload_sts2_with_expired_time(self, expired_time):
        actions = ["vod:ApplyUploadInfo", "vod:CommitUploadInfo"]
        resources = []
        statement = Statement.new_allow_statement(actions, resources)
        inline_policy = Policy([statement])
        return self.sign_sts2(inline_policy, expired_time)

    def get_upload_sts2(self):
        return self.get_upload_sts2_with_expired_time(60 * 60)

    def upload_material(self, request):
        oid, session_key, avg_speed = self.upload_tob(request.SpaceName, request.FilePath, request.FileType,
                                                              request.FileName)

        req = VodCommitUploadInfoRequest()
        req.SpaceName = request.SpaceName
        req.SessionKey = session_key
        req.Functions = request.Functions
        req.CallbackArgs = request.CallbackArgs

        resp = self.commit_upload_info(req)
        if resp.ResponseMetadata.Error.Code != '':
            print(resp.ResponseMetadata.RequestId)
            raise Exception(resp.ResponseMetadata.Error)
        return resp

    #
    # GetAllPlayInfo.
    #
    # @param request VodGetAllPlayInfoRequest
    # @return VodGetAllPlayInfoResponse
    # @raise Exception
    def get_all_play_info(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetAllPlayInfo", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetAllPlayInfoResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetAllPlayInfoResponse(), True)

    #
    # GetPlayInfo.
    #
    # @param request VodGetPlayInfoRequest
    # @return VodGetPlayInfoResponse
    # @raise Exception
    def get_play_info(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetPlayInfo", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetPlayInfoResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetPlayInfoResponse(), True)

    #
    # GetPrivateDrmPlayAuth.
    #
    # @param request VodGetPrivateDrmPlayAuthRequest
    # @return VodGetPrivateDrmPlayAuthResponse
    # @raise Exception
    def get_private_drm_play_auth(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetPrivateDrmPlayAuth", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetPrivateDrmPlayAuthResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetPrivateDrmPlayAuthResponse(), True)

    #
    # GetHlsDecryptionKey.
    #
    # @param request VodGetHlsDecryptionKeyRequest
    # @return VodGetHlsDecryptionKeyResponse
    # @raise Exception
    def get_hls_decryption_key(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetHlsDecryptionKey", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetHlsDecryptionKeyResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetHlsDecryptionKeyResponse(), True)

    #
    # GetPlayInfoWithLiveTimeShiftScene.
    #
    # @param request VodGetPlayInfoWithLiveTimeShiftSceneRequest
    # @return VodGetPlayInfoWithLiveTimeShiftSceneResponse
    # @raise Exception
    def get_play_info_with_live_time_shift_scene(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetPlayInfoWithLiveTimeShiftScene", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetPlayInfoWithLiveTimeShiftSceneResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetPlayInfoWithLiveTimeShiftSceneResponse(), True)

    #
    # UploadMediaByUrl.
    #
    # @param request VodUrlUploadRequest
    # @return VodUrlUploadResponse
    # @raise Exception
    def upload_media_by_url(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("UploadMediaByUrl", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodUrlUploadResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodUrlUploadResponse(), True)

    #
    # QueryUploadTaskInfo.
    #
    # @param request VodQueryUploadTaskInfoRequest
    # @return VodQueryUploadTaskInfoResponse
    # @raise Exception
    def query_upload_task_info(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("QueryUploadTaskInfo", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodQueryUploadTaskInfoResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodQueryUploadTaskInfoResponse(), True)

    #
    # ApplyUploadInfo.
    #
    # @param request VodApplyUploadInfoRequest
    # @return VodApplyUploadInfoResponse
    # @raise Exception
    def apply_upload_info(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ApplyUploadInfo", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodApplyUploadInfoResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodApplyUploadInfoResponse(), True)

    #
    # CommitUploadInfo.
    #
    # @param request VodCommitUploadInfoRequest
    # @return VodCommitUploadInfoResponse
    # @raise Exception
    def commit_upload_info(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("CommitUploadInfo", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodCommitUploadInfoResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodCommitUploadInfoResponse(), True)

    #
    # UpdateMediaInfo.
    #
    # @param request VodUpdateMediaInfoRequest
    # @return VodUpdateMediaInfoResponse
    # @raise Exception
    def update_media_info(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("UpdateMediaInfo", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodUpdateMediaInfoResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodUpdateMediaInfoResponse(), True)

    #
    # UpdateMediaPublishStatus.
    #
    # @param request VodUpdateMediaPublishStatusRequest
    # @return VodUpdateMediaPublishStatusResponse
    # @raise Exception
    def update_media_publish_status(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("UpdateMediaPublishStatus", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodUpdateMediaPublishStatusResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodUpdateMediaPublishStatusResponse(), True)

    #
    # GetMediaInfos.
    #
    # @param request VodGetMediaInfosRequest
    # @return VodGetMediaInfosResponse
    # @raise Exception
    def get_media_infos(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetMediaInfos", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetMediaInfosResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetMediaInfosResponse(), True)

    #
    # GetRecommendedPoster.
    #
    # @param request VodGetRecommendedPosterRequest
    # @return VodGetRecommendedPosterResponse
    # @raise Exception
    def get_recommended_poster(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetRecommendedPoster", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetRecommendedPosterResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetRecommendedPosterResponse(), True)

    #
    # DeleteMedia.
    #
    # @param request VodDeleteMediaRequest
    # @return VodDeleteMediaResponse
    # @raise Exception
    def delete_media(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("DeleteMedia", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodDeleteMediaResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodDeleteMediaResponse(), True)

    #
    # DeleteTranscodes.
    #
    # @param request VodDeleteTranscodesRequest
    # @return VodDeleteTranscodesResponse
    # @raise Exception
    def delete_transcodes(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("DeleteTranscodes", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodDeleteTranscodesResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodDeleteTranscodesResponse(), True)

    #
    # GetMediaList.
    #
    # @param request VodGetMediaListRequest
    # @return VodGetMediaListResponse
    # @raise Exception
    def get_media_list(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetMediaList", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetMediaListResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetMediaListResponse(), True)

    #
    # GetSubtitleInfoList.
    #
    # @param request VodGetSubtitleInfoListRequest
    # @return VodGetSubtitleInfoListResponse
    # @raise Exception
    def get_subtitle_info_list(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetSubtitleInfoList", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetSubtitleInfoListResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetSubtitleInfoListResponse(), True)

    #
    # UpdateSubtitleStatus.
    #
    # @param request VodUpdateSubtitleStatusRequest
    # @return VodUpdateSubtitleStatusResponse
    # @raise Exception
    def update_subtitle_status(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("UpdateSubtitleStatus", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodUpdateSubtitleStatusResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodUpdateSubtitleStatusResponse(), True)

    #
    # UpdateSubtitleInfo.
    #
    # @param request VodUpdateSubtitleInfoRequest
    # @return VodUpdateSubtitleInfoResponse
    # @raise Exception
    def update_subtitle_info(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("UpdateSubtitleInfo", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodUpdateSubtitleInfoResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodUpdateSubtitleInfoResponse(), True)

    #
    # GetAuditFramesForAudit.
    #
    # @param request VodGetAuditFramesForAuditRequest
    # @return VodGetAuditFramesForAuditResponse
    # @raise Exception
    def get_audit_frames_for_audit(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetAuditFramesForAudit", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetAuditFramesForAuditResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetAuditFramesForAuditResponse(), True)

    #
    # GetMLFramesForAudit.
    #
    # @param request VodGetMLFramesForAuditRequest
    # @return VodGetMLFramesForAuditResponse
    # @raise Exception
    def get_m_l_frames_for_audit(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetMLFramesForAudit", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetMLFramesForAuditResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetMLFramesForAuditResponse(), True)

    #
    # GetBetterFramesForAudit.
    #
    # @param request VodGetBetterFramesForAuditRequest
    # @return VodGetBetterFramesForAuditResponse
    # @raise Exception
    def get_better_frames_for_audit(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetBetterFramesForAudit", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetBetterFramesForAuditResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetBetterFramesForAuditResponse(), True)

    #
    # GetAudioInfoForAudit.
    #
    # @param request VodGetAudioInfoForAuditRequest
    # @return VodGetAudioInfoForAuditResponse
    # @raise Exception
    def get_audio_info_for_audit(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetAudioInfoForAudit", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetAudioInfoForAuditResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetAudioInfoForAuditResponse(), True)

    #
    # GetAutomaticSpeechRecognitionForAudit.
    #
    # @param request VodGetAutomaticSpeechRecognitionForAuditRequest
    # @return VodGetAutomaticSpeechRecognitionForAuditResponse
    # @raise Exception
    def get_automatic_speech_recognition_for_audit(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetAutomaticSpeechRecognitionForAudit", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetAutomaticSpeechRecognitionForAuditResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetAutomaticSpeechRecognitionForAuditResponse(), True)

    #
    # GetAudioEventDetectionForAudit.
    #
    # @param request VodGetAudioEventDetectionForAuditRequest
    # @return VodGetAudioEventDetectionForAuditResponse
    # @raise Exception
    def get_audio_event_detection_for_audit(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetAudioEventDetectionForAudit", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetAudioEventDetectionForAuditResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetAudioEventDetectionForAuditResponse(), True)

    #
    # CreateVideoClassification.
    #
    # @param request VodCreateVideoClassificationRequest
    # @return VodCreateVideoClassificationResponse
    # @raise Exception
    def create_video_classification(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("CreateVideoClassification", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodCreateVideoClassificationResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodCreateVideoClassificationResponse(), True)

    #
    # UpdateVideoClassification.
    #
    # @param request VodUpdateVideoClassificationRequest
    # @return VodUpdateVideoClassificationResponse
    # @raise Exception
    def update_video_classification(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("UpdateVideoClassification", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodUpdateVideoClassificationResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodUpdateVideoClassificationResponse(), True)

    #
    # DeleteVideoClassification.
    #
    # @param request VodDeleteVideoClassificationRequest
    # @return VodDeleteVideoClassificationResponse
    # @raise Exception
    def delete_video_classification(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("DeleteVideoClassification", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodDeleteVideoClassificationResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodDeleteVideoClassificationResponse(), True)

    #
    # ListVideoClassifications.
    #
    # @param request VodListVideoClassificationsRequest
    # @return VodListVideoClassificationsResponse
    # @raise Exception
    def list_video_classifications(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListVideoClassifications", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodListVideoClassificationsResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodListVideoClassificationsResponse(), True)

    #
    # ListSnapshots.
    #
    # @param request VodListSnapshotsRequest
    # @return VodListSnapshotsResponse
    # @raise Exception
    def list_snapshots(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListSnapshots", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodListSnapshotsResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodListSnapshotsResponse(), True)

    #
    # StartWorkflow.
    #
    # @param request VodStartWorkflowRequest
    # @return VodStartWorkflowResponse
    # @raise Exception
    def start_workflow(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("StartWorkflow", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodStartWorkflowResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodStartWorkflowResponse(), True)

    #
    # RetrieveTranscodeResult.
    #
    # @param request VodRetrieveTranscodeResultRequest
    # @return VodRetrieveTranscodeResultResponse
    # @raise Exception
    def retrieve_transcode_result(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("RetrieveTranscodeResult", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodRetrieveTranscodeResultResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodRetrieveTranscodeResultResponse(), True)

    #
    # CreateSpace.
    #
    # @param request VodCreateSpaceRequest
    # @return VodCreateSpaceResponse
    # @raise Exception
    def create_space(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("CreateSpace", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodCreateSpaceResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodCreateSpaceResponse(), True)

    #
    # ListSpace.
    #
    # @param request VodListSpaceRequest
    # @return VodListSpaceResponse
    # @raise Exception
    def list_space(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListSpace", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodListSpaceResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodListSpaceResponse(), True)

    #
    # GetSpaceDetail.
    #
    # @param request VodGetSpaceDetailRequest
    # @return VodGetSpaceDetailResponse
    # @raise Exception
    def get_space_detail(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetSpaceDetail", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetSpaceDetailResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetSpaceDetailResponse(), True)

    #
    # UpdateSpace.
    #
    # @param request VodUpdateSpaceRequest
    # @return VodUpdateSpaceResponse
    # @raise Exception
    def update_space(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("UpdateSpace", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodUpdateSpaceResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodUpdateSpaceResponse(), True)

    #
    # UpdateSpaceUploadConfig.
    #
    # @param request VodUpdateSpaceUploadConfigRequest
    # @return VodUpdateSpaceUploadConfigResponse
    # @raise Exception
    def update_space_upload_config(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("UpdateSpaceUploadConfig", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodUpdateSpaceUploadConfigResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodUpdateSpaceUploadConfigResponse(), True)

    #
    # ListDomain.
    #
    # @param request VodListDomainRequest
    # @return VodListDomainResponse
    # @raise Exception
    def list_domain(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListDomain", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodListDomainResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodListDomainResponse(), True)

    #
    # CreateCdnRefreshTask.
    #
    # @param request VodCreateCdnRefreshTaskRequest
    # @return VodCreateCdnRefreshTaskResponse
    # @raise Exception
    def create_cdn_refresh_task(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("CreateCdnRefreshTask", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodCreateCdnRefreshTaskResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodCreateCdnRefreshTaskResponse(), True)

    #
    # CreateCdnPreloadTask.
    #
    # @param request VodCreateCdnPreloadTaskRequest
    # @return VodCreateCdnPreloadTaskResponse
    # @raise Exception
    def create_cdn_preload_task(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("CreateCdnPreloadTask", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodCreateCdnPreloadTaskResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodCreateCdnPreloadTaskResponse(), True)

    #
    # ListCdnTasks.
    #
    # @param request VodListCdnTasksRequest
    # @return VodListCdnTasksResponse
    # @raise Exception
    def list_cdn_tasks(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListCdnTasks", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodListCdnTasksResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodListCdnTasksResponse(), True)

    #
    # ListCdnAccessLog.
    #
    # @param request VodListCdnAccessLogRequest
    # @return VodListCdnAccessLogResponse
    # @raise Exception
    def list_cdn_access_log(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListCdnAccessLog", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodListCdnAccessLogResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodListCdnAccessLogResponse(), True)

    #
    # ListCdnTopAccessUrl.
    #
    # @param request VodListCdnTopAccessUrlRequest
    # @return VodListCdnTopAccessUrlResponse
    # @raise Exception
    def list_cdn_top_access_url(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListCdnTopAccessUrl", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodListCdnTopAccessUrlResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodListCdnTopAccessUrlResponse(), True)

    #
    # DescribeVodDomainBandwidthData.
    #
    # @param request VodDescribeVodDomainBandwidthDataRequest
    # @return VodDescribeVodDomainBandwidthDataResponse
    # @raise Exception
    def describe_vod_domain_bandwidth_data(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("DescribeVodDomainBandwidthData", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodDescribeVodDomainBandwidthDataResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodDescribeVodDomainBandwidthDataResponse(), True)

    #
    # ListCdnUsageData.
    #
    # @param request VodListCdnUsageDataRequest
    # @return VodCdnStatisticsCommonResponse
    # @raise Exception
    def list_cdn_usage_data(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListCdnUsageData", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodCdnStatisticsCommonResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodCdnStatisticsCommonResponse(), True)

    #
    # ListCdnStatusData.
    #
    # @param request VodListCdnStatusDataRequest
    # @return VodCdnStatisticsCommonResponse
    # @raise Exception
    def list_cdn_status_data(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListCdnStatusData", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodCdnStatisticsCommonResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodCdnStatisticsCommonResponse(), True)

    #
    # DescribeIpInfo.
    #
    # @param request VodDescribeIPInfoRequest
    # @return VodDescribeIPInfoResponse
    # @raise Exception
    def describe_ip_info(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("DescribeIpInfo", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodDescribeIPInfoResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodDescribeIPInfoResponse(), True)

    #
    # ListCdnPvData.
    #
    # @param request VodListCdnPvDataRequest
    # @return VodCdnStatisticsCommonResponse
    # @raise Exception
    def list_cdn_pv_data(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("ListCdnPvData", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodCdnStatisticsCommonResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodCdnStatisticsCommonResponse(), True)

    #
    # AddCallbackSubscription.
    #
    # @param request VodAddCallbackSubscriptionRequest
    # @return VodAddCallbackSubscriptionResponse
    # @raise Exception
    def add_callback_subscription(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("AddCallbackSubscription", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodAddCallbackSubscriptionResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodAddCallbackSubscriptionResponse(), True)

    #
    # SetCallbackEvent.
    #
    # @param request VodSetCallbackEventRequest
    # @return VodSetCallbackEventResponse
    # @raise Exception
    def set_callback_event(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("SetCallbackEvent", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodSetCallbackEventResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodSetCallbackEventResponse(), True)

    #
    # GetSmartStrategyLitePlayInfo.
    #
    # @param request VodGetSmartStrategyLitePlayInfoRequest
    # @return VodGetSmartStrategyLitePlayInfoResponse
    # @raise Exception
    def get_smart_strategy_lite_play_info(self, request):
        try:
            if sys.version_info[0] == 3:
                jsonData = MessageToJson(request, False, True)
                params = json.loads(jsonData)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            else:
                params = MessageToDict(request, False, True)
                for k, v in params.items():
                    if isinstance(v, (int, float, bool, str, unicode)) is True:
                        continue
                    else:
                        params[k] = json.dumps(v)
            res = self.get("GetSmartStrategyLitePlayInfo", params)
        except Exception as Argument:
            try:
                resp = Parse(Argument.__str__(), VodGetSmartStrategyLitePlayInfoResponse(), True)
            except Exception:
                raise Argument
            else:
                raise Exception(resp.ResponseMetadata.Error.Code)
        else:
            return Parse(res, VodGetSmartStrategyLitePlayInfoResponse(), True)




#coding: utf-8

import json
import os
import redo
import threading

import minio
from requests import exceptions

from volcengine.ApiInfo import ApiInfo
from volcengine.Credentials import Credentials
from volcengine.base.Service import Service
from volcengine.ServiceInfo import ServiceInfo

ORDER_TYPE_MAP = {
    'ascend': 1,
    'descend': 2
}

ALLOW_FILE_TYPES = [
    'asset',
    'scene_file',
    'client_script'
]


class VerenderService(Service):
    _instance_lock = threading.Lock()
    _version = "2021-12-31"
    _DEFAULT_REGION = "cn-north-1"
    _SERVICE = "verender"

    def __new__(cls, *args, **kwargs):
        if not hasattr(VerenderService, '_instance'):
            with VerenderService._instance_lock:
                if not hasattr(VerenderService, '_instance'):
                    VerenderService._instance = object.__new__(cls)

        return VerenderService._instance

    def __init__(self, ak=None, sk=None):
        if ak and sk:
            self.set_ak(ak)
            self.set_sk(sk)

        self.service_info = VerenderService.get_service_info()
        self.api_info_map = VerenderService.get_api_info()
        super(VerenderService, self).__init__(self.service_info, self.api_info_map)

    @staticmethod
    def get_service_info():
        service_info = ServiceInfo(
            'open.volcengineapi.com',
            {'Accept': 'application/json'},
            Credentials('', '', VerenderService._SERVICE, VerenderService._DEFAULT_REGION),
            5,
            5,
            'https'
        )

        return service_info

    @staticmethod
    def get_api_info():
        api_info_map = {
            'ListWorkspaces': ApiInfo(
                'GET',
                '/',
                {'Action': 'ListWorkspaces', 'Version': VerenderService._version},
                {},
                {}
            ),
            'PurchaseWorkspace': ApiInfo(
                'POST',
                '/',
                {'Action': 'PurchaseWorkspace', 'Version': VerenderService._version},
                {},
                {}
            ),
            'DeleteWorkspace': ApiInfo(
                'POST',
                '/',
                {'Action': 'DeleteWorkspace', 'Version': VerenderService._version},
                {},
                {}
            ),
            'UpdateWorkspace': ApiInfo(
                'POST',
                '/',
                {'Action': 'UpdateWorkspace', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetStorageAccess': ApiInfo(
                'GET',
                '/',
                {'Action': 'GetStorageAccess', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetWorkspaceLimit': ApiInfo(
                'GET',
                '/',
                {'Action': 'GetWorkspaceLimit', 'Version': VerenderService._version},
                {},
                {}
            ),
            'ListResourcePools': ApiInfo(
                'GET',
                '/',
                {'Action': 'ListResourcePools', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetWorkspaceHardwareSpecifications': ApiInfo(
                'GET',
                '/',
                {'Action': 'GetWorkspaceHardwareSpecifications', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetAccountStatistics': ApiInfo(
                'POST',
                '/',
                {'Action': 'GetAccountStatistics', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetAccountStatisticDetails': ApiInfo(
                'POST',
                '/',
                {'Action': 'GetAccountStatisticDetails', 'Version': VerenderService._version},
                {},
                {}
            ),
            'DownloadStatisticDetails': ApiInfo(
                'POST',
                '/',
                {'Action': 'DownloadStatisticDetails', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetCurrentUser': ApiInfo(
                'GET',
                '/',
                {'Action': 'GetCurrentUser', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetJobOverallStatistics': ApiInfo(
                'GET',
                '/',
                {'Action': 'GetJobOverallStatistics', 'Version': VerenderService._version},
                {},
                {}
            ),
            'CreateRenderJob': ApiInfo(
                'POST',
                '/',
                {'Action': 'CreateRenderJob', 'Version': VerenderService._version},
                {},
                {}
            ),
            'ListRenderJobs': ApiInfo(
                'GET',
                '/',
                {'Action': 'ListRenderJobs', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetRenderJob': ApiInfo(
                'GET',
                '/',
                {'Action': 'GetRenderJob', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetFrames': ApiInfo(
                'GET',
                '/',
                {'Action': 'GetFrames', 'Version': VerenderService._version},
                {},
                {}
            ),
            'GetLayerFrames': ApiInfo(
                'POST',
                '/',
                {'Action': 'GetLayerFrames', 'Version': VerenderService._version},
                {},
                {}
            ),
            'UpdateRenderJobPriority': ApiInfo(
                'POST',
                '/',
                {'Action': 'UpdateRenderJobPriority', 'Version': VerenderService._version},
                {},
                {}
            ),
            'StartRenderJob': ApiInfo(
                'POST',
                '/',
                {'Action': 'StartRenderJob', 'Version': VerenderService._version},
                {},
                {}
            ),
            'PauseRenderJob': ApiInfo(
                'POST',
                '/',
                {'Action': 'PauseRenderJob', 'Version': VerenderService._version},
                {},
                {}
            ),
            'StopRenderJob': ApiInfo(
                'POST',
                '/',
                {'Action': 'StopRenderJob', 'Version': VerenderService._version},
                {},
                {}
            ),
            'DeleteRenderJob': ApiInfo(
                'POST',
                '/',
                {'Action': 'DeleteRenderJob', 'Version': VerenderService._version},
                {},
                {}
            ),
            'FullSpeedRenderJob': ApiInfo(
                'POST',
                '/',
                {'Action': 'FullSpeedRenderJob', 'Version': VerenderService._version},
                {},
                {}
            ),
            'RetryJob': ApiInfo(
                'POST',
                '/',
                {'Action': 'RetryJob', 'Version': VerenderService._version},
                {},
                {}
            ),
            'EditRenderJob': ApiInfo(
                'POST',
                '/',
                {'Action': 'EditRenderJob', 'Version': VerenderService._version},
                {},
                {}
            ),
            'PauseJobs': ApiInfo(
                'POST',
                '/',
                {'Action': 'PauseJobs', 'Version': VerenderService._version},
                {},
                {}
            ),
            'ResumeJobs': ApiInfo(
                'POST',
                '/',
                {'Action': 'ResumeJobs', 'Version': VerenderService._version},
                {},
                {}
            ),
            'StopJobs': ApiInfo(
                'POST',
                '/',
                {'Action': 'StopJobs', 'Version': VerenderService._version},
                {},
                {}
            ),
            'DeleteJobs': ApiInfo(
                'POST',
                '/',
                {'Action': 'DeleteJobs', 'Version': VerenderService._version},
                {},
                {}
            ),
            'FullSpeedRenderJobs': ApiInfo(
                'POST',
                '/',
                {'Action': 'FullSpeedRenderJobs', 'Version': VerenderService._version},
                {},
                {}
            ),
            'ListMyMessages': ApiInfo(
                'GET',
                '/',
                {'Action': 'ListMyMessages', 'Version': VerenderService._version},
                {},
                {}
            ),
            'MarkMessageAsRead': ApiInfo(
                'POST',
                '/',
                {'Action': 'MarkMessageAsRead', 'Version': VerenderService._version},
                {},
                {}
            ),
            'BatchMarkMessagesAsRead': ApiInfo(
                'POST',
                '/',
                {'Action': 'BatchMarkMessagesAsRead', 'Version': VerenderService._version},
                {},
                {}
            ),
            'MarkAllMessagesAsRead': ApiInfo(
                'POST',
                '/',
                {'Action': 'MarkAllMessagesAsRead', 'Version': VerenderService._version},
                {},
                {}
            ),
            'DeleteMessage': ApiInfo(
                'POST',
                '/',
                {'Action': 'DeleteMessage', 'Version': VerenderService._version},
                {},
                {}
            ),
            'BatchDeleteMessages': ApiInfo(
                'POST',
                '/',
                {'Action': 'BatchDeleteMessages', 'Version': VerenderService._version},
                {},
                {}
            ),
            'DeleteAllMessages': ApiInfo(
                'POST',
                '/',
                {'Action': 'DeleteAllMessages', 'Version': VerenderService._version},
                {},
                {}
            ),
            'DeleteAllReadMessages': ApiInfo(
                'POST',
                '/',
                {'Action': 'DeleteAllReadMessages', 'Version': VerenderService._version},
                {},
                {}
            )
        }

        return api_info_map

    @staticmethod
    def get_minio_client(endpoint, ak, sk, token):
        addr = endpoint
        secure = True
        if endpoint[:8] == "https://":
            addr = endpoint[8:]
        elif endpoint[:7] == "http://":
            addr = endpoint[7:]
            secure = False

        return minio.Minio(
            addr,
            access_key=ak,
            secret_key=sk,
            session_token=token,
            secure=secure,
            region=VerenderService._DEFAULT_REGION
        )

    @redo.retriable(retry_exceptions=(exceptions.ConnectionError, exceptions.ConnectTimeout),
                    sleeptime=0.1, jitter=0.01, attempts=2)
    def upload_file(self, workspace_id, file_list, file_type="asset"):
        if file_type not in ALLOW_FILE_TYPES:
            raise TypeError("file type must be one of {}".format(ALLOW_FILE_TYPES))

        store_info = self.get_storage_access(workspace_id)

        try:
            endpoint = store_info['Result']['cluster_endpoint']
            ak = store_info['Result']['access_key']
            sk = store_info['Result']['secret_key']
            token = store_info['Result']['bucket_token']
            bucket = store_info['Result']['bucket_name']
        except Exception as ex:
            raise Exception('get storage access for {} error {}'.format(workspace_id, str(ex)))

        objects = []
        cli = VerenderService.get_minio_client(endpoint, ak, sk, token)
        for file_path in file_list:
            object = os.path.basename(file_path)
            cli.fput_object(bucket, object, file_path)
            objects.append(object)

        return objects


    @redo.retriable(retry_exceptions=(exceptions.ConnectionError, exceptions.ConnectTimeout),
                    sleeptime=0.1, jitter=0.01, attempts=2)
    def _call_json(self, api_name, param=None, body=None):
        api_info = self.api_info_map.get(api_name)

        if not api_info:
            raise Exception('Api named {} not found.'.format(api_name))

        param = param or {}
        body = json.dumps(body) or ''
        method = {
            'POST': self.json,
            'GET': self.get
        }.get(api_info.method, self.post)

        try:
            res = method(api_name, param, body)
        except Exception as ex:
            raise Exception(str(ex))

        if '' == res:
            raise Exception('Empty response.')

        try:
            res_json = json.loads(res)
        except Exception as ex:
            raise Exception('Invalid json format {}.'.format(str(ex)))

        if 'Error' in res_json.get('ResponseMetadata', {}):
            err = res_json['ResponseMetadata']['Error']
            raise Exception('{} {} {}'.format(
                err.get('CodeN', 0),
                err.get('Code', 'Unknown error'),
                err.get('Message', 'Backend service return an error, but no message were found.')
            ))

        return res_json

    @redo.retriable(retry_exceptions=(exceptions.ConnectionError, exceptions.ConnectTimeout),
                    sleeptime=0.1, jitter=0.01, attempts=2)
    def _call_download(self, api_name, save_file, param=None, body=None):
        api_info = self.api_info_map.get(api_name)

        if not api_info:
            raise Exception('Api named {} not found'.format(api_name))

        param = param or {}
        body = body or {}
        method = {
            'GET': self.get,
            'POST': self.post
        }.get(api_info.method, self.post)

        try:
            res = method(api_name, param, body)
        except Exception as ex:
            raise Exception(str(ex))

        with open(save_file, 'w+') as f:
            f.write(res)

    # start_purchase_time 和 end_purchase_time格式应符合RFC3339, 比如 2022-04-01T00:00:00+08:00
    def list_workspaces(self, page_num=1, page_size=10, list_scope=None, workspace_id=None,
                        workspace_name=None, start_purchase_time=None, end_purchase_time=None,
                        order_type=None, orderby=None, fuzzy_workspace_name=None,
                        fuzzy_search_content=None):
        param = {
            'PageNum': page_num,
            'PageSize': page_size
        }

        if list_scope:
            param['ListScope'] = list_scope

        if workspace_id:
            param['WorkspaceId'] = workspace_id

        if workspace_name:
            param['WorkspaceName'] = workspace_name

        if start_purchase_time:
            param['StartPurchaseTime'] = start_purchase_time

        if end_purchase_time:
            param['EndPurchaseTime'] = end_purchase_time

        if order_type:
            param['OrderType'] = order_type

        if orderby:
            param['OrderBy'] = orderby

        if fuzzy_workspace_name:
            param['FuzzyWorkspaceName'] = fuzzy_workspace_name

        if fuzzy_search_content:
            param['FuzzySearchContent'] = fuzzy_search_content

        return self._call_json('ListWorkspaces', param)

    # @param storage_total: 工作区的存储空间上限，默认100G，最大1000G
    # @param resource_pool_id: 可用的ResourcePool通过get_resource_pool接口获取
    # @param cps_id: 可用的CPS集群通过get_resource_pool接口获取
    def purchase_workspace(self, name, description, storage_total, resource_pool_id, cps_id):
        data = {
            'Name': name,
            'Description': description,
            'StorageTotal': storage_total,
            'ResourcePoolId': resource_pool_id,
            'CpsId': cps_id
        }

        return self._call_json('PurchaseWorkspace', {}, body=data)

    def delete_workspace(self, workspace_id):
        param = {
            'WorkspaceId': workspace_id
        }

        return self._call_json('DeleteWorkspace', param)

    def update_workspace(self, workspace_id, name, description, storage_total):
        param = {
            'WorkspaceId': workspace_id
        }

        data = {
            'Name': name,
            'Description': description,
            'StorageTotal': storage_total
        }

        return self._call_json('UpdateWorkspace', param, body=data)

    def get_storage_access(self, workspace_id):
        param = {
            'WorkspaceId': workspace_id
        }

        return self._call_json('GetStorageAccess', param)

    # 返回当前账号可创建的最大工作区数目和单个工作区可申请的最大存储空间
    def get_workspace_limit(self):
        return self._call_json('GetWorkspaceLimit')

    # 返回当前账号可用的{ResourcePool, CPS}列表
    def list_resource_pools(self):
        return self._call_json('ListResourcePools')

    def get_workspace_hardware_specifications(self, workspace_id):
        param = {
            'WorkspaceId': workspace_id
        }

        return self._call_json('GetWorkspaceHardwareSpecifications', param)

    # @ param time_type 目前支持 last_day, last_week, last_month, last_year, all
    # @ param start_time 和 end_time 当前不生效
    def get_account_statistics(self, time_type, start_time=None, end_time=None, workspace_ids=None, user_ids=None):
        param = {
            'TimeType': time_type
        }

        if start_time:
            param['StartTime'] = start_time

        if end_time:
            param['EndTime'] = end_time

        data = {}
        if workspace_ids:
            data['WorkspaceIds'] = workspace_ids

        if user_ids:
            data['UserIds'] = user_ids

        return self._call_json('GetAccountStatistics', param, body=data)

    # @param start_time和end_time的格式需符合RFC3339, 比如 2022-05-01T00:00:00+08:00
    # @param order_type, descend表示降序, ascend表示升序
    # @param order_by, 指明排序的列名, 支持 StartTime 和 TotalCost，默认为 StartTime
    # @param job_types，指明渲染任务类别列表，支持 maya_redshift, maya_arnold, c4d_redshift, blender, octane
    def get_account_statistic_detail(self, start_time, end_time, page_num=1, page_size=10, order_type="descend",
                                    order_by='StartTime', workspace_ids=None, user_ids=None, job_types=None, keyword=None):
        param = {
            'StartTime': start_time,
            'EndTime': end_time,
            'PageNum': page_num,
            'PageSize': page_size,
            'OrderField': order_by
        }

        if order_type in ORDER_TYPE_MAP:
            param['OrderBy'] = ORDER_TYPE_MAP[order_type]

        data = {}
        if workspace_ids:
            data['WorkspaceIds'] = workspace_ids

        if user_ids:
            data['UserIds'] = user_ids

        if job_types:
            data['JobTypes'] = job_types

        if keyword:
            data['Keyword'] = keyword

        return self._call_json('GetAccountStatisticDetails', param, body=data)

    # 将指定条件的统计明细以csv文件的形式返还，save_file指明要保存的文件路径，其他参数参考 get_statistic_details
    def download_statistic_details(self, start_time, end_time, save_file, page_num=1, page_size=10, order_type='descend',
                                    order_by='StartTime', workspace_ids=None, user_ids=None, job_types=None, keyword=None):
        param = {
            'StartTime': start_time,
            'EndTime': end_time,
            'PageNum': page_num,
            'PageSize': page_size,
            'OrderField': order_by
        }

        if order_type in ORDER_TYPE_MAP:
            param['OrderBy'] = ORDER_TYPE_MAP[order_type]

        data = {}
        if workspace_ids:
            data['WorkspaceIds'] = workspace_ids

        if user_ids:
            data['UserIds'] = user_ids

        if job_types:
            data['JobTypes'] = job_types

        if keyword:
            data['Keyword'] = keyword

        return self._call_download('DownloadStatisticDetails', save_file, param, body=data)

    # 获取当前登录用户的详细信息
    def get_current_user(self):
        return self._call_json('GetCurrentUser')

    # 获取当前登录用户提交的渲染任务的概要信息
    def get_job_overall_statistics(self):
        return self._call_json('GetJobOverallStatistics')

    # render_job_cfg = {
    #     'Username': 'username', --username 从接口 get_current_user 获取
    #     'Title': 'Test-Job-1', --Title可以多个任务重复
    #     'Description': '',
    #     'DccTool': 'maya', --支持maya, c4d, blender
    #     'DccVersion': '',
    #     'Renderer': 'redshift', --maya支持redshift/arnold, c4d支持cycles, blender支持redshift
    #     'RendererVersion': '',
    #     'Tryout': true, --true表示先试渲染, false表示正式渲染
    #     'TryoutFrames': ['1', '2', '3'], --试渲染的帧列表, Tryout为true时才有意义, 类型为string
    #     'MayaProjectPath': '', --maya工程里mel文件的路径, 需要先上传到存储，非必选
    #     'SceneFile': 'test_legency_render.ma', --渲染工程的路径，需要先上传到存储后再提交渲染任务
    #     'OutputFormat': 'PNG', --渲染结果格式, 支持PNG和EXR
    #     'Resolutions': [
    #         {
    #             'Width': 100,
    #             'Height': 100
    #         }
    #     ], --各渲染层输出结果的分辨率, 非必选
    #     'PathMapping': {
    #         'C:/': ''
    #     }, --将WINDOWS下的路径映射为LINUX下的路径, 非必选
    #     'FrameSettings': {
    #         'Start': 1,
    #         'End': 10,
    #         'Step': 1
    #     }, --渲染帧信息，非必选, 默认是{0, 1, 1}
    #     'Cameras': ['top', 'side'], --摄像机列表, 非必选
    #     'Layers': ['layer-1'], --渲染层列表, 非必选
    #     'TimeoutReminder': 36000, --超时提醒时间, 单位为分钟, 非必选
    #     'TimeoutStopper': 36000, --超时停止时间, 单位为分钟, 非必选
    #     'OutputImageTemple': {
    #         'padding': 4, --帧号的位数，如0000, 0001
    #         'ImageNameTemple': 'template', --输出结果的模版
    #         'SceneName': 'test', --渲染场景的名称
    #         'Extension': 'ext' --渲染结果的后缀
    #     }, --渲染结果的命名规则，非必选
    #     'WantedCellSpecs': [
    #         {
    #             'ComputerResourceType': 'CPU', --支持CPU/T4, arnold使用CPU, 其他渲染器是用GPU, 建议不配置, 系统自行决定
    #             'ComputerResourceCount': 10
    #         }
    #     ], --用户选择的执行渲染的硬件规格, 非必选
    #     'UseLegacyRenderLayers': true --maya渲染工程是否使用传统层, 非必选
    # }
    def create_render_job(self, workspace_id, render_job_cfg):
        param = {
            'WorkspaceId': workspace_id
        }

        return self._call_json('CreateRenderJob', param, body=render_job_cfg)

    # @param order_by表示结果的排序类型, ascend表示升序, 其他表示降序, 默认为降序, 按照任务的创建时间?
    # @param tryout为0表示正式渲染, 其他为试渲染
    # @param status表示要查询的任务状态, 多个之间使用逗号分隔, 支持Empty, Starting, Queuing, Analyzing, Processing,
    #           Rendered, Cleanup, Paused, Deleted, Stopped, Finished, Error
    # @param job_type表示渲染任务类型, 多个之间使用逗号分隔, 支持maya_redshift, maya_arnold, c4d_redshift, blender_cycle
    def list_render_jobs(self, workspace_id, order_by='descend', page_num=1, page_size=10, tryout=0, status=None, job_type=None):
        param = {
            'WorkspaceId': workspace_id,
            'PageNum': page_num,
            'PageSize': page_size,
            'Tryout': tryout
        }

        if order_by in ORDER_TYPE_MAP:
            param['OrderBy'] = ORDER_TYPE_MAP[order_by]

        if status:
            param['Status'] = status

        # TODO: JobType拆分为 Dcc 和 Render
        if job_type:
            param['JobType'] = job_type

        return self._call_json('ListRenderJobs', param)

    def get_render_job(self, workspace_id, render_job_id):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id
        }

        return self._call_json('GetRenderJob', param)

    def get_frames(self, workspace_id, render_job_id, page_num=1, page_size=10):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id,
            'PageNum': page_num,
            'PageSize': page_size
        }

        return self._call_json('GetFrames', param)

    # layer_requests = {
    #   'LayerRequests': [
    #       {
    #           'LayerIndex': 0,
    #           'PageNum': 1,
    #           'PageSize': 10,
    #           'Statuses': [0, 1]
    #       }
    #   ]
    # }
    def get_layer_frames(self, workspace_id, render_job_id, layer_requests):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id
        }

        return self._call_json('GetLayerFrames', param, body=layer_requests)

    def update_render_job_priority(self, workspace_id, render_job_id, priority):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id,
            'Priority': priority
        }

        return self._call_json('UpdateRenderJobPriority', param)

    def start_render_job(self, workspace_id, render_job_id):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id
        }

        return self._call_json('StartRenderJob', param)

    def pause_render_job(self, workspace_id, render_job_id):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id
        }

        return self._call_json('PauseRenderJob', param)

    def stop_render_job(self, workspace_id, render_job_id):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id
        }

        return self._call_json('StopRenderJob', param)

    def delete_render_job(self, workspace_id, render_job_id):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id
        }

        return self._call_json('DeleteRenderJob', param)

    def full_speed_render_job(self, workspace_id, render_job_id):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id
        }

        return self._call_json('FullSpeedRenderJob', param)

    def retry_render_job(self, workspace_id, render_job_id):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id
        }

        return self._call_json('RetryJob', param)

    def edit_render_job(self, workspace_id, render_job_id, title, description):
        param = {
            'WorkspaceId': workspace_id,
            'RenderJobId': render_job_id
        }

        data = {}
        if title:
            data['Title'] = title

        if description:
            data['Description'] = description

        return self._call_json('EditRenderJob', param, body=data)

    def pause_jobs(self, workspace_id, render_job_ids):
        param = {
            'WorkspaceId': workspace_id
        }

        data = {
            'JobIds': render_job_ids
        }

        return self._call_json('PauseJobs', param, body=data)

    def resume_jobs(self, workspace_id, render_job_ids):
        param = {
            'WorkspaceId': workspace_id
        }

        data = {
            'JobIds': render_job_ids
        }

        return self._call_json('ResumeJobs', param, body=data)

    def stop_jobs(self, workspace_id, render_job_ids):
        param = {
            'WorkspaceId': workspace_id
        }

        data = {
            'JobIds': render_job_ids
        }

        return self._call_json('StopJobs', param, body=data)

    def delete_jobs(self, workspace_id, render_job_ids):
        param = {
            'WorkspaceId': workspace_id
        }

        data = {
            'JobIds': render_job_ids
        }

        return self._call_json('DeleteJobs', param, body=data)

    def full_speed_render_jobs(self, workspace_id, render_job_ids):
        param = {
            'WorkspaceId': workspace_id
        }

        data = {
            'JobIds': render_job_ids
        }

        return self._call_json('FullSpeedRenderJobs', param, body=data)

    def list_my_messages(self, page_num=1, page_size=10):
        param = {
            'PageNum': page_num,
            'PageSize': page_size
        }

        return self._call_json('ListMyMessages', param)

    def mark_message_as_read(self, message_id):
        param = {
            'MessageId': message_id
        }

        return self._call_json('MarkMessageAsRead', param)

    def batch_mark_messages_as_read(self, message_ids):
        data = {
            'MessageIds': message_ids
        }

        return self._call_json('BatchMarkMessagesAsRead', {}, body=data)

    def mark_all_messages_as_read(self):
        return self._call_json('MarkAllMessagesAsRead', {})

    def delete_message(self, message_id):
        param = {
            'MessageId': message_id
        }

        return self._call_json('DeleteMessage', param)

    def batch_delete_messages(self, message_ids):
        data = {
            'MessageIds': message_ids
        }

        return self._call_json('BatchDeleteMessages', {}, body=data)

    def delete_all_messages(self):
        return self._call_json('DeleteAllMessages', {})

    def delete_all_read_messages(self):
        return self._call_json('DeleteAllReadMessages', {})


#  -*- coding: utf-8 -*-
import datetime

from volcengine.example.cdn import ak, sk
from volcengine.cdn.service import CDNService

if __name__ == '__main__':
    svc = CDNService()
    svc.set_ak(ak)
    svc.set_sk(sk)
    now = int(datetime.datetime.now().strftime("%s"))
    body = {
        'StartTime': now - 86400,
        'EndTime': now,
        'Metric': 'pv',
        'Domain': 'example.com',
        'Item': 'region'
    }
    print(body)

    resp = svc.describe_cdn_domain_top_data(body)
    print(resp)
